@@ -1,4 +1,4 @@
-package MBX::Alien::FLTK::Base;
+package inc::MBX::Alien::FLTK::Base;
 {
     use strict;
     use warnings;
@@ -8,21 +8,22 @@ package MBX::Alien::FLTK::Base;
     use File::Find qw[find];
     use Carp qw[carp];
     use base 'Module::Build';
-    use lib qw[../../../../inc];
-    use MBX::Alien::FLTK::Utility
-        qw[_o _a _dir _file _rel _abs _exe find_h find_lib can_run];
+    use inc::MBX::Alien::FLTK::Utility
+        qw[_o _a _path _dir _file _rel _abs _exe find_h find_lib can_run];
+    use lib _abs('.');
 
     sub fltk_dir {
-        my ($self) = @_;
-        my $return = $self->notes('fltk_dir');
-        return $return if $return;
-        $self->notes(
-                'fltk_dir' =>
-                    _abs(sprintf '%s/src/fltk-%s-r%d', $self->base_dir(),
-                         $self->notes('fltk_branch'), $self->notes('fltk_svn')
-                    )
-        );
-        return _dir($self->notes('fltk_dir'));
+        my ($self, $extra) = @_;
+        $self->depends_on('extract_fltk');
+        return
+            _abs(_path($self->notes('extract'),
+                       (      'fltk-'
+                            . $self->notes('branch') . '-r'
+                            . $self->notes('svn')
+                       ),
+                       $extra || ()
+                 )
+            );
     }
 
     sub archive {
@@ -30,7 +31,7 @@ package MBX::Alien::FLTK::Base;
         my $arch = $args->{'output'};
         my @cmd = ($self->notes('AR'), $arch, @{$args->{'objects'}});
         print STDERR "@cmd\n" if !$self->quiet;
-        return MBX::Alien::FLTK::Utility::run(@cmd);
+        return inc::MBX::Alien::FLTK::Utility::run(@cmd) ? $arch : ();
     }
 
     sub test_exe {
@@ -44,6 +45,8 @@ package MBX::Alien::FLTK::Base;
 
     sub compile {
         my ($self, $args) = @_;
+        local $^W = 0;
+        local $self->cbuilder->{'quiet'} = 1;
         my $code = 0;
         if (!$args->{'source'}) {
             (my $FH, $args->{'source'}) = tempfile(
@@ -81,6 +84,8 @@ package MBX::Alien::FLTK::Base;
 
     sub link_exe {
         my ($self, $args) = @_;
+        local $^W = 0;
+        local $self->cbuilder->{'quiet'} = 1;
         my $exe = eval {
             $self->cbuilder->link_executable(
                                      objects            => $args->{'objects'},
@@ -111,29 +116,12 @@ package MBX::Alien::FLTK::Base;
         return $exe;
     }
 
-    sub _archdir {
-        my ($self, $p) = @_;
-        my ($vol, $dir, $file) = File::Spec->splitpath($p || '');
-        return
-            File::Spec->catfile($self->install_destination('arch'),
-                                qw[Alien FLTK], File::Spec->splitdir($dir),
-                                $file);
-    }
-
     sub ACTION_copy_headers {
         my ($self) = @_;
-        $self->depends_on('fetch_fltk');
-        $self->depends_on('extract_fltk');
-        $self->depends_on('configure_fltk');
         $self->depends_on('write_config_h');
-        $self->depends_on('build_fltk');
-        my $headers_location =
-            _dir($self->fltk_dir(),
-                 ($self->notes('fltk_branch') eq '1.3.x'
-                  ? 'FL'
-                  : 'fltk'
-                 )
-            );
+        my $headers_location
+            = _path($self->fltk_dir(), $self->notes('headers_path'));
+        my $headers_share = _path($self->base_dir(), qw[share include]);
         if (!chdir $headers_location) {
             printf 'Failed to cd to %s to copy headers', $headers_location;
             exit 0;
@@ -142,62 +130,38 @@ package MBX::Alien::FLTK::Base;
             wanted => sub {
                 return if -d;
                 $self->copy_if_modified(
-                            from => $File::Find::name,
-                            to   => _dir(
-                                       $self->base_dir(),
-                                       qw[blib arch Alien FLTK include],
-                                       'fltk-' . $self->notes('fltk_branch'),
-                                       ($self->notes('fltk_branch') eq '1.3.x'
-                                        ? 'FL'
-                                        : 'fltk'
-                                       ),
-                                       $File::Find::name
-                            )
+                                     from => $File::Find::name,
+                                     to   => _path(
+                                                 $headers_share,
+                                                 $self->notes('headers_path'),
+                                                 $File::Find::name
+                                     )
                 );
             },
             no_chdir => 1
             },
             '.';
-        if (!chdir _dir($self->fltk_dir())) {
+        if (!chdir _path($self->fltk_dir())) {
             print 'Failed to cd to fltk\'s include directory';
             exit 0;
         }
         $self->copy_if_modified(from => 'config.h',
-                                to =>
-                                    _dir(
-                                        $self->base_dir(),
-                                        qw[blib arch Alien FLTK include],
-                                        'fltk-' . $self->notes('fltk_branch'),
-                                        'config.h'
-                                    )
-        );
-        print "Installing headers...\n" if !$self->quiet;
+                                to   => _path($headers_share, 'config.h'));
+        print "Copying headers to sharedir...\n" if !$self->quiet;
         if (!chdir $self->base_dir()) {
             printf 'Failed to return to %s', $self->base_dir();
             exit 0;
         }
-        $self->notes(headers => $self->_archdir('include'));
+        $self->notes(headers => $headers_share);
         return 1;
     }
 
     # Configure
     sub configure {
         my ($self, $args) = @_;
-        $self->notes('ldflags' => ($self->notes('fltk_branch') eq '1.3.x'
-                                   ? ' -lfltk '
-                                   : ' -lfltk2 '
-                     )
-        );
-        $self->notes('_a'       => $Config{'_a'});
-        $self->notes('cxxflags' => ' ');
-        $self->notes('GL'       => ' ');
-        $self->notes(
-            'image_flags' => (
-                $self->notes('fltk_branch') eq '1.3.x'
-                ? ' -lfltk_images -lfltk_png -lfltk_z -lfltk_images -lfltk_jpeg '
-                : ' -lfltk2_images -lfltk2_png -lfltk2_z -lfltk2_images -lfltk2_jpeg '
-            )
-        );
+        $self->notes('_a'            => $Config{'_a'});
+        $self->notes('cxxflags'      => ' ');
+        $self->notes('GL'            => ' ');
         $self->notes('include_dirs'  => {});
         $self->notes('library_paths' => {});
         $self->notes(
@@ -218,32 +182,30 @@ package MBX::Alien::FLTK::Base;
                 USE_X11_MULTITHREADING => 0,                           # 2.0
                 USE_XFT                => 0,                           # both
                 USE_XCURSOR            => undef,
-                USE_CAIRO =>
-                    ($self->notes('fltk_branch') eq '2.0.x' ? 0 : undef)
-                ,                                                      # both
-                USE_CLIPOUT      => 0,
-                USE_XSHM         => 0,
-                HAVE_XDBE        => 0,                                  # both
-                USE_XDBE         => 'HAVE_XDBE',                        # both
-                HAVE_OVERLAY     => 0,                                  # both
-                USE_OVERLAY      => 0,
-                USE_XINERAMA     => 0,
-                USE_MULTIMONITOR => 1,
-                USE_STOCK_BRUSH  => 1,
-                USE_XIM          => 1,
-                HAVE_ICONV       => 0,
-                HAVE_GL          => (find_h('gl.h') ? 1 : undef),       # both
-                HAVE_GL_GLU_H    => (find_h('gl/glu.h') ? 1 : undef),   # both
-                HAVE_GL_OVERLAY  => 'HAVE_OVERLAY',                     # both
-                USE_GL_OVERLAY   => 0,                                  # 2.0
-                USE_GLEW         => 0,                                  # 2.0
-                HAVE_GLXGETPROCADDRESSARB => undef,                     # 1.3
+                USE_CAIRO              => $self->notes('use_cairo'),   # both
+                USE_CLIPOUT            => 0,
+                USE_XSHM               => 0,
+                HAVE_XDBE              => 0,                           # both
+                USE_XDBE               => 'HAVE_XDBE',                 # both
+                HAVE_OVERLAY           => 0,                           # both
+                USE_OVERLAY            => 0,
+                USE_XINERAMA           => 0,
+                USE_MULTIMONITOR       => 1,
+                USE_STOCK_BRUSH        => 1,
+                USE_XIM                => 1,
+                HAVE_ICONV             => 0,
+                HAVE_GL => (find_h('gl.h') ? 1 : undef),               # both
+                HAVE_GL_GLU_H => (find_h('gl/glu.h') ? 1 : undef),     # both
+                HAVE_GL_OVERLAY           => 'HAVE_OVERLAY',           # both
+                USE_GL_OVERLAY            => 0,                        # 2.0
+                USE_GLEW                  => 0,                        # 2.0
+                HAVE_GLXGETPROCADDRESSARB => undef,                    # 1.3
                 HAVE_DIRENT_H => (find_h('dirent.h') ? 1 : undef),
                 HAVE_STRING_H       => (find_h('string.h')       ? 1 : undef),
                 HAVE_SYS_NSTRING_H  => (find_h('sys/ndir.h')     ? 1 : undef),
                 HAVE_SYS_DIR_H      => (find_h('sys/dir.h')      ? 1 : undef),
                 HAVE_NDIR_H         => (find_h('ndir.h')         ? 1 : undef),
-                HAVE_SCANDIR        => 1,
+                HAVE_SCANDIR        => 0,
                 HAVE_SCANDIR_POSIX  => undef,
                 HAVE_STRING_H       => (find_h('string.h')       ? 1 : undef),
                 HAVE_STRINGS_H      => (find_h('strings.h')      ? 1 : undef),
@@ -389,134 +351,23 @@ int main ( ) {
         }
         return 1;
     }
-    {
-        my %___LIBS = ();
-
-        sub LIBS {
-            my ($self) = @_;
-            %___LIBS = (
-                fltk2_images => {
-                    directory => 'images',
-                    source    => [
-                        map { $_ . '.cxx' }
-                            qw[FileIcon2 Fl_Guess_Image fl_jpeg fl_png
-                            HelpDialog images_core pnmImage xpmFileImage]
-                    ]
-                },
-                fltk2_z => {
-                    directory => 'images/zlib',
-                    source    => [
-                        map { $_ . '.c' }
-                            qw[adler32 compress crc32 gzio uncompr deflate
-                            trees zutil inflate inftrees inffast infblock
-                            infcodes infutil]
-                    ]
-                },
-                fltk2_jpeg => {
-                    directory => 'images/libjpeg',
-                    source    => [
-                        map { $_ . '.c' }
-                            qw[jmemnobs jcapimin jcapistd jccoefct jccolor
-                            jcdctmgr jchuff jcinit jcmainct jcmarker jcmaster
-                            jcomapi jcparam jcphuff jcprepct jcsample jctrans
-                            jdapimin jdapistd jdatadst jdatasrc jdcoefct jdcolor
-                            jddctmgr jdhuff jdinput jdmainct jdmarker jdmaster
-                            jdmerge jdphuff jdpostct jdsample jdtrans jerror
-                            jfdctflt jfdctfst jfdctint jidctflt jidctfst jidctint
-                            jidctred jquant1 jquant2 jutils jmemmgr]
-                    ]
-                },
-                fltk2_png => {
-                    directory => 'images/libpng',
-                    include   => 'zlib',
-                    source    => [
-                        map { $_ . '.c' }
-                            qw[png pngset pngget pngrutil pngtrans pngwutil
-                            pngread pngrio pngwio pngwrite pngrtran pngwtran
-                            pngmem pngerror pngpread]
-                    ]
-                },
-                fltk2 => {
-                    directory => 'src',
-                    source    => [
-                        (map { $_ . '.c' } qw[scandir string utf vsnprintf]),
-                        (  map { $_ . '.cxx' }
-                               qw[add_idle addarc addcurve Adjuster AlignGroup
-                               AnsiWidget args BarGroup bmpImage Browser
-                               Browser_load Button CheckButton Choice clip
-                               Clock Color color_chooser ComboBox compose
-                               Cursor CycleButton default_glyph Dial
-                               DiamondBox dnd drawtext EngravedLabel error
-                               event_key_state file_chooser FileBrowser
-                               FileChooser FileChooser2 FileIcon FileInput
-                               filename_absolute filename_ext filename_isdir
-                               filename_list filename_match filename_name
-                               fillrect Fl_Menu_Item FloatInput fltk_theme
-                               Font gifImage Group GSave HelpView
-                               HighlightButton Image Input InputBrowser
-                               InvisibleWidget Item key_name LightButton
-                               list_fonts load_plugin lock Menu Menu_add
-                               Menu_global Menu_popup MenuBar MenuWindow
-                               message MultiImage NumericInput numericsort
-                               Output OvalBox overlay_rect own_colormap
-                               PackedGroup path PlasticBox PopupMenu
-                               Preferences ProgressBar RadioButton readimage
-                               RepeatButton ReturnButton RoundBox RoundedBox
-                               run Scrollbar ScrollGroup scrollrect setcolor
-                               setdisplay setvisual ShadowBox ShapedWindow
-                               SharedImage ShortcutAssignment show_colormap
-                               Slider StatusBarGroup StringList Style StyleSet
-                               Symbol SystemMenuBar TabGroup TabGroup2
-                               TextBuffer TextDisplay TextEditor ThumbWheel
-                               TiledGroup TiledImage Tooltip UpBox Valuator
-                               ValueInput ValueOutput ValueSlider Widget
-                               Widget_draw WidgetAssociation Window
-                               Window_fullscreen Window_hotspot Window_iconize
-                               WizardGroup xbmImage xpmImage]
-                        )
-                    ]
-                },
-                fltk2_gl => {
-                    directory => 'OpenGL',
-                    source    => [
-                        map { $_ . '.cxx' }
-                            qw[Fl_Gl_Choice Fl_Gl_Overlay Fl_Gl_Window gl_draw
-                            gl_start]
-                    ]
-                },
-                fltk2_glut => {
-                      directory => 'glut',
-                      source    => [
-                          map { $_ . '.cxx' } qw[glut_compatability glut_font]
-                      ]
-                }
-            ) if !keys %___LIBS;
-            return \%___LIBS;
-        }
-    }
 
     sub build_fltk {
         my ($self, $build) = @_;
         $self->quiet(1);
-        my (@lib, @_libs);
-        if ($self->notes('fltk_branch') eq '1.3.x') {
-            @_libs = grep { $_ !~ m[2] } keys %{$self->LIBS};
-        }
-        elsif ($self->notes('fltk_branch') eq '2.0.x') {
-            @_libs = grep { $_ =~ m[2] } keys %{$self->LIBS};
-        }
-        for my $lib (sort { lc $a cmp lc $b } @_libs) {
+        $self->notes('libs' => []);
+        my $libs = $self->notes('libs_source');
+        for my $lib (sort { lc $a cmp lc $b } keys %$libs) {
             print "Building $lib...\n";
-            if (!chdir _dir($build->fltk_dir(),
-                            $self->LIBS->{$lib}{'directory'}))
-            {   printf 'Cannot chdir to %s',
-                    _dir($build->fltk_dir(),
-                         $self->LIBS->{$lib}{'directory'});
+            if (!chdir _path($build->fltk_dir(), $libs->{$lib}{'directory'}))
+            {   printf 'Cannot chdir to %s to build %s',
+                    _path($build->fltk_dir(), $libs->{$lib}{'directory'}),
+                    $lib;
                 exit 0;
             }
             my @obj;
-            for my $src (@{$self->LIBS->{$lib}{'source'}}) {
-                my $obj = _o($src);
+            for my $src (sort { lc $a cmp lc $b } @{$libs->{$lib}{'source'}})
+            {   my $obj = _o($src);
                 $obj
                     = $build->up_to_date($src, $obj)
                     ? $obj
@@ -524,25 +375,23 @@ int main ( ) {
                     print "Compiling $src...\n";
                     return
                         $self->compile(
-                        {source       => $src,
-                         include_dirs => [
-                             $Config{'incpath'},
-                             '.',
-                             _rel($build->fltk_dir($build)),
-                             _rel($build->fltk_dir($build) . '/fltk/compat/'),
-                             _rel($build->fltk_dir($build)
-                                      . (
-                                        $self->notes('fltk_branch') eq '1.3.x'
-                                        ? ''
-                                        : '/images'
-                                      )
-                                      . '/zlib/'
-                             ),
-                             (keys %{$self->notes('include_dirs')})
-                         ],
-                         cxxflags => [$Config{'ccflags'}],
-                         output   => $obj
-                        }
+                          {source       => $src,
+                           include_dirs => [
+                               $Config{'incpath'},
+                               $build->fltk_dir(),
+                               $build->fltk_dir($self->notes('headers_path')),
+                               $build->fltk_dir(
+                                    $self->notes('include_path_compatability')
+                               ),
+                               $build->fltk_dir(
+                                           $self->notes('include_path_images')
+                                               . '/zlib/'
+                               ),
+                               (keys %{$self->notes('include_dirs')})
+                           ],
+                           cxxflags => [$Config{'ccflags'}, '-MD'],
+                           output   => $obj
+                          }
                         );
                     }
                     ->();
@@ -552,7 +401,7 @@ int main ( ) {
                 }
                 push @obj, $obj;
             }
-            my $_lib = _rel(_dir($build->fltk_dir, 'lib', _a($lib)));
+            my $_lib = _rel($build->fltk_dir('lib/' . _a($lib)));
             $lib
                 = $build->up_to_date(\@obj, $_lib)
                 ? $_lib
@@ -564,96 +413,215 @@ int main ( ) {
                 printf 'Failed to create %s library', $lib;
                 exit 0;
             }
-            push @lib, _abs($lib);
+            push @{$self->notes('libs')}, _abs($lib);
         }
-        if (!chdir $build->fltk_dir($self)) {
-            print 'Failed to cd to ' . $self->fltk_dir();
+        if (!chdir $build->fltk_dir()) {
+            print 'Failed to cd to ' . $self->fltk_dir() . ' to return home';
             exit 0;
         }
-        $self->quiet(0);
-        return @lib ? 1 : 0;
+        return scalar @{$self->notes('libs')};
     }
 
     # Module::Build actions
     sub ACTION_fetch_fltk {
-        my ($self) = @_;
-        my %mirrors = (
-                'ftp.easysw.com'  => 'California, USA',
-                'ftp2.easysw.com' => 'New Jersey, USA',
-                'ftp.funet.fi/pub/mirrors/ftp.easysw.com' => 'Espoo, Finland',
-                'ftp.rz.tu-bs.de/pub/mirror/ftp.easysw.com/ftp' =>
-                    'Braunschweig, Germany'
-        );
-        my $dest = 'snapshots';
-        $self->notes('archive_path' =>
-                         _abs(_dir(sprintf('snapshots/fltk-%s-r%s.tar.gz',
-                                           $self->notes('fltk_branch'),
-                                           $self->notes('fltk_svn')
-                                   )
-                              )
-                         )
-        );
-        return if -f $self->notes('archive_path');
+        my ($self, %args) = @_;
         require File::Fetch;
-        my $path;
-    MIRROR: for my $mirror (keys %mirrors) {
 
-            for my $prot (qw[ftp http]) {
-                my $from
-                    = sprintf '%s://%s/pub/fltk/snapshots/fltk-%s-r%s.tar.gz',
-                    $prot, $mirror,
-                    $self->notes('fltk_branch'),
-                    $self->notes('fltk_svn');
-                printf
-                    "Fetching r%s from the %s branch of fltk from %s mirror\n    %s...\n",
-                    $self->notes('fltk_svn'), $self->notes('fltk_branch'),
-                    $mirrors{$mirror}, $from;
-                $path = File::Fetch->new(uri => $from)->fetch(to => $dest);
+        #
+        $File::Fetch::TIMEOUT = $File::Fetch::TIMEOUT = 45;    # Be quick
+        $args{'to'} ||= _abs($self->notes('snapshot_dir'));
 
-                # XXX - verify with md5
-                last MIRROR if $path;
+        #
+        printf "Fetching SVN snapshot %d... ", $self->notes('svn');
+        my ($schemes, $exts, %mirrors)
+            = ([qw[http ftp]], [qw[gz bz2]], _snapshot_mirrors());
+        my ($attempt, $total)
+            = (0, scalar(@$schemes) * scalar(@$exts) * scalar(keys %mirrors));
+        my $mirrors = [keys %mirrors];
+        {                                                      # F-Y shuffle
+            my $i = @$mirrors;
+            while (--$i) {
+                my $j = int rand($i + 1);
+                @$mirrors[$i, $j] = @$mirrors[$j, $i];
             }
         }
-        if (!$path) {
-            my $msg = sprintf <<'END',
-We failed to fetch fltk-%s-r%s.tar.gz.
-
-If this problem persists, you may download the archive yourself and put it in
-the ./%s/ directory. Alien::FLTK will attempt to extract and build the libs
-from there.
-
-Use any of these mirrors:
-
-END
-                $self->notes('fltk_branch'), $self->notes('fltk_svn'), $dest;
-            for my $mirror (keys %mirrors) {
-                $msg .= " " x 4 . $mirrors{$mirror} . "\n";
-                for my $prot (qw[ftp http]) {
-                    $msg .= sprintf
-                        "      %s://%s/pub/fltk/snapshots/fltk-%s-r%s.tar.gz\n",
-                        $prot, $mirror, $self->notes('fltk_branch'),
-                        $self->notes('fltk_svn');
+        my ($dir, $archive, $extention);
+    MIRROR: for my $mirror (@$mirrors) {
+        EXT: for my $ext (@$exts) {
+                {
+                    my $to =
+                        _abs(sprintf '%s/fltk-%s-r%d.tar.%s',
+                             $args{'to'},
+                             $self->notes('branch'),
+                             $self->notes('svn'),
+                             $ext
+                        );
+                    if (-f $to) {
+                        printf
+                            "(Found preexisting snapshot) ",
+                            $to,
+                            -s $to;
+                        $dir       = $args{'to'};
+                        $archive   = $to;
+                        $extention = $ext;
+                        last MIRROR;
+                    }
+                }
+            SCHEME: for my $scheme (@$schemes) {
+                    printf "\n[%d/%d] Trying %s mirror based in %s... ",
+                        ++$attempt, $total, uc $scheme, $mirror;
+                    my $ff =
+                        File::Fetch->new(
+                              uri => sprintf
+                                  '%s://%s/fltk/snapshots/fltk-%s-r%d.tar.%s',
+                              $scheme, $mirrors{$mirror},
+                              $self->notes('branch'),
+                              $self->notes('svn'), $ext
+                        );
+                    $archive = $ff->fetch(to => $args{'to'});
+                    if ($archive and -f $archive) {
+                        $self->notes('snapshot_mirror_uri'      => $ff->uri);
+                        $self->notes('snapshot_mirror_location' => $mirror);
+                        $archive = _abs(sprintf '%s/fltk-%s-r%d.tar.%s',
+                                        $args{'to'}, $self->notes('branch'),
+                                        $self->notes('svn'), $ext);
+                        $extention = $ext;
+                        $dir       = $args{'to'};
+                        last MIRROR;
+                    }
                 }
             }
-            push @{$self->notes('errors')},
-                {stage   => 'fltk source download',
-                 fatal   => 1,
-                 message => $msg
-                };
-            return;
         }
-        return $path;
+        if (!$archive) {    # bad news
+            my (@urls, $i);
+            for my $ext (@$exts) {
+                for my $mirror (sort values %mirrors) {
+                    for my $scheme (@$schemes) {
+                        push @urls,
+                            sprintf
+                            '[%d] %s://%s/fltk/snapshots/fltk-%s-r%d.tar.%s',
+                            ++$i, $scheme, $mirror,
+                            $self->notes('branch'),
+                            $self->notes('svn'), $ext;
+                    }
+                }
+            }
+            my $urls = join "\n", @urls;
+            push @{$self->notes('errors')},
+                {stage => 'fltk source download',
+                 fatal => 1,
+                 message =>
+                     sprintf
+                     <<'END', _abs($self->notes('snapshot_dir')), $urls};
+Okay, we just failed at life.
+
+If you want, you may manually download a snapshot and place it in
+    %s
+
+Please, use one of the following mirrors:
+%s
+
+Exiting...
+END
+            $self->dispatch('check_errors');
+        }
+        print "done.\n";
+        $self->notes('snapshot_dir' => $args{'to'});
+        {
+            require Digest::MD5;
+            print 'Validating archive... ';
+            my $FH;
+            if (!open($FH, '<', $archive)) {
+                push @{$self->notes('errors')},
+                    {stage => 'fltk source validation',
+                     fatal => 1,
+                     message =>
+                         "Can't open '$archive' to check MD5 checksum: $!"
+                    };
+                $self->dispatch('check_errors');
+
+                # XXX - Should I delete the archive and retry?
+            }
+            binmode($FH);
+            unshift @INC, _abs(_path($self->base_dir, 'lib'));
+            if (eval 'require ' . $self->module_name) {
+                my $md5 = $self->module_name->md5;
+                if (Digest::MD5->new->addfile($FH)->hexdigest eq
+                    $md5->{$extention})
+                {   print "MD5 checksum is okay\n";
+                    last;
+                }
+            }
+            else {
+                print "Cannot find checksum. Hope this works out...\n";
+                last;
+            }
+            shift @INC;
+            close $FH;
+            if ($self->notes('bad_fetch_retry')->{'count'}++ > 10) {
+                push @{$self->notes('errors')},
+                    {
+                    stage => 'fltk source validation',
+                    fatal => 1,
+                    message =>
+                        'Found/downloaded archive failed to match MD5 checksum... Giving up.'
+                    };
+                $self->dispatch('check_errors');
+            }
+            push @{$self->notes('errors')},
+                {
+                stage => 'fltk source validation',
+                fatal => 0,
+                message =>
+                    'Found/downloaded archive failed to match MD5 checksum... Retrying.'
+                };
+            $self->dispatch('check_errors');
+
+            #unlink $archive;
+            $self->dispatch('fetch_fltk');
+        }
+        $self->notes('snapshot_path' => $archive);
+        $self->notes('snapshot_dir'  => $dir);       # Unused but good to know
+             #$self->add_to_cleanup($dir);
+        return 1;
+    }
+
+    sub _snapshot_mirrors {
+        return (
+            'California, USA' => 'ftp.easysw.com/pub',
+            'New Jersey, USA' => 'ftp2.easysw.com/pub',
+            'Espoo, Finland' => 'ftp.funet.fi/pub/mirrors/ftp.easysw.com/pub',
+            'Braunschweig, Germany' =>
+                'ftp.rz.tu-bs.de/pub/mirror/ftp.easysw.com/ftp/pub'
+        );
     }
 
     sub ACTION_extract_fltk {
-        my ($self) = @_;
+        my ($self, %args) = @_;
         $self->depends_on('fetch_fltk');
-        my $archive = $self->notes('archive_path');
-        return 1 if -d _dir($self->fltk_dir());
-        printf 'Extracting fltk source from %s... ', _rel($archive);
         require Archive::Extract;
-        my $ae = Archive::Extract->new(archive => $archive);
-        if (!$ae->extract(to => 'src')) {
+        $args{'from'} ||= $self->notes('snapshot_path');
+        $args{'to'}   ||= _abs($self->notes('extract_dir'));
+        if (-d _abs($args{'to'} . sprintf '/fltk-%s-r%d',
+                    $self->notes('branch'),
+                    $self->notes('svn')
+            )
+            )
+        {   $self->notes('extract' => $args{'to'});
+            return 1;    # XXX - what should we do?!?
+            require File::Path;
+            printf "Removing existing directory...\n", $args{'to'};
+            File::Path::remove_tree(_abs($args{'to'} . sprintf '/fltk-%s-r%d',
+                                         $self->notes('branch'),
+                                         $self->notes('svn')
+                                    )
+            );
+        }
+        my $ae = Archive::Extract->new(archive => $args{'from'});
+        printf 'Extracting %s to %s... ', _rel($args{'from'}),
+            _rel($args{'to'});
+        my $okay = $ae->extract(to => $args{'to'});
+        if (!$okay) {
             push @{$self->notes('errors')},
                 {stage   => 'fltk source extraction',
                  fatal   => 1,
@@ -661,17 +629,21 @@ END
                 };
             return;
         }
-        print "okay\n";
+        $self->notes('extract'       => $args{'to'});
+        $self->notes('snapshot_path' => $args{'from'})
+            ;    # If used from commandline
+        $self->add_to_cleanup($ae->extract_path);
+        print "done.\n";
         return 1;
     }
 
     sub ACTION_configure_fltk {
         my ($self) = @_;
-        $self->depends_on('fetch_fltk');
         $self->depends_on('extract_fltk');
 
-        #if (!$self->notes('config') || !keys %{$self->notes('config')}) {
-        if (!$self->notes('config') || !-f $self->fltk_dir() . '/config.h') {
+        #if (   !$self->notes('config')
+        #    || !-f $self->fltk_dir('config.h'))
+        {
             print "Gathering configuration data...\n";
             $self->configure();
             $self->notes(timestamp_configure => time);
@@ -681,11 +653,14 @@ END
 
     sub ACTION_write_config_h {
         my ($self) = @_;
-        $self->depends_on('fetch_fltk');
-        $self->depends_on('extract_fltk');
+        return 1
+            if -f $self->notes('config_path')
+                && -s $self->notes('config_path');
         $self->depends_on('configure_fltk');
         if (!chdir $self->fltk_dir()) {
-            print 'Failed to cd to ' . $self->fltk_dir();
+            print 'Failed to cd to '
+                . $self->fltk_dir()
+                . ' to find config.h';
             exit 0;
         }
         if (   (!-f 'config.h')
@@ -696,7 +671,9 @@ END
         {   {
                 print 'Creating config.h... ';
                 if (!chdir $self->fltk_dir()) {
-                    print 'Failed to cd to ' . $self->fltk_dir();
+                    print 'Failed to cd to '
+                        . $self->fltk_dir()
+                        . ' to write config.h';
                     exit 0;
                 }
                 my $config = '';
@@ -732,37 +709,24 @@ END
                 print "okay\n";
             }
         }
-        {    # Ganked from Module::Build::Notes
-            print 'Updating Alien::FLTK config... ';
-            my $me = _abs($self->base_dir() . '/lib/Alien/FLTK.pm');
-            require IO::File;
-            my $mode_orig = (stat $me)[2] & 07777;
-            chmod($mode_orig | 0222, $me);    # Make it writeable
-            my $fh = IO::File->new($me, 'r+')
-                or die "Can't rewrite $me: $!";
-            seek($fh, 0, 0);
-            while (<$fh>) { last if /^__DATA__$/; }
-
-            if (eof($fh)) {    #warn "Couldn't find __DATA__ token in $me";
-                $fh->print("\n__DATA__\n");
+        {
+            require YAML::Tiny;
+            printf 'Updating %s config... ', $self->module_name;
+            my $me        = $self->notes('config_path');
+            my $mode_orig = 0644;
+            if (!-d _dir($me)) {
+                require File::Path;
+                File::Path::make_path(_dir($me));
             }
-            seek($fh, tell($fh), 0);
-            my $data = $self->notes();
-            if (eval 'require Data::Dump') {
-                $fh->print(sprintf 'do{ my $x = %s; $x; }' . "\n",
-                           Data::Dump::pp($data));
+            elsif (-d $me) {
+                $mode_orig = (stat $me)[2] & 07777;
+                chmod($mode_orig | 0222, $me);    # Make it writeable
             }
-            else {
-                require Data::Dumper;
-                my $Dumper = Data::Dumper->new([$data], ['x']);
-                $Dumper->Purity(1);
-                $fh->print(sprintf 'do{ my %s; $x; }' . "\n",
-                           $Dumper->Dump());
-            }
-            truncate($fh, tell($fh));
-            $fh->close;
+            my $yaml = YAML::Tiny->new;
+            $yaml->[0] = $self->notes();
+            $yaml->write($me);
             chmod($mode_orig, $me)
-                or warn "Couldn't restore permissions on $me: $!";
+                or printf 'Cannot restore permissions on %s: %s', $me, $!;
             print "okay\n";
         }
         if (!chdir $self->base_dir()) {
@@ -774,30 +738,16 @@ END
 
     sub ACTION_clear_config {
         my ($self) = @_;
-        print 'Cleaning Alien::FLTK config... ';
-        my $me = _abs($self->base_dir() . '/lib/Alien/FLTK.pm');
-        require IO::File;
+        printf 'Cleaning %s config... ', $self->module_name();
+        my $me        = $self->notes('config_path');
         my $mode_orig = (stat $me)[2] & 07777;
         chmod($mode_orig | 0222, $me);    # Make it writeable
-        my $fh = IO::File->new($me, 'r+')
-            or die "Can't rewrite $me: $!";
-        seek($fh, 0, 0);
-        while (<$fh>) { last if /^__DATA__$/; }
-        die "Couldn't find __DATA__ token in $me" if eof($fh);
-        seek($fh, tell($fh), 0);
-        $fh->print("do{ my \$x = { }; \$x; }\n");
-        truncate($fh, tell($fh));
-        $fh->close;
-        chmod($mode_orig, $me)
-            or warn "Couldn't restore permissions on $me: $!";
+        unlink $me;
         print "okay\n";
     }
 
     sub ACTION_build_fltk {
         my ($self) = @_;
-        $self->depends_on('fetch_fltk');
-        $self->depends_on('extract_fltk');
-        $self->depends_on('configure_fltk');
         $self->depends_on('write_config_h');
         if (!chdir $self->fltk_dir()) {
             printf 'Failed to cd to %s to locate libs libs',
@@ -809,26 +759,16 @@ END
             printf 'Failed to return to %s to copy libs', $self->base_dir();
             exit 0;
         }
-        if (!chdir _dir($self->fltk_dir() . '/lib')) {
+        if (!chdir _path($self->fltk_dir() . '/lib')) {
             printf 'Failed to cd to %s to copy libs', $self->fltk_dir();
             exit 0;
         }
-        $self->copy_if_modified(from => $_,
-                                to_dir =>
-                                    _dir($self->base_dir(),
-                                         qw[blib arch Alien FLTK libs],
-                                         'fltk-' . $self->notes('fltk_branch')
-                                    )
-            )
-            for grep defined, map { my $_a = _a($_); -f $_a ? $_a : () } (
-            $self->notes('fltk_branch') eq '1.3.x' ? (
-               qw?fltk     fltk_gl   fltk_glut fltk_forms
-                   fltk_images fltk_jpeg fltk_png  fltk_z?
-                )
-            : (qw?fltk2     fltk2_gl   fltk2_glut fltk2_forms
-                   fltk2_images fltk2_jpeg fltk2_png  fltk2_z?
-            )
+        for my $lib (@{$self->notes('libs')}) {
+            $self->copy_if_modified(
+                            from   => $lib,
+                            to_dir => _path($self->base_dir(), qw[share libs])
             );
+        }
         if (!chdir $self->base_dir()) {
             print 'Failed to cd to base directory';
             exit 0;
@@ -838,10 +778,8 @@ END
 
     sub ACTION_code {
         my ($self) = @_;
-        for my $action (
-            qw[fetch_fltk extract_fltk configure_fltk write_config_h build_fltk
-            copy_headers])
-        {   $self->depends_on($action);
+        for my $action (qw[build_fltk copy_headers]) {
+            $self->depends_on($action);
             $self->dispatch('check_errors');
         }
         return $self->SUPER::ACTION_code;
@@ -860,6 +798,7 @@ END
                 ($error->{'fatal'} ? ('*** Fatal') : 'Non-fatal'),
                 $error->{'stage'}, $msg, '-- ' x 10;
         }
+        $self->notes('errors' => []);
         if ($fatal) {
             printf STDOUT ('*** ' x 15) . "\n"
                 . '%s fatal error%s encountered during the build process. '
